  //  Copyright (c) Arthritis Research UK Centre for Epidemiology, University of Manchester (2016-2018)

  // Decision 9: what to do if two prescriptions overlap

  // Handle overlapping prescriptions
  // options are:	9a  do nothing, allow prescriptions to overlap (implicity sum doses)
  //              9b  move later prescription to next available time that this product is not prescribed

  args dec9

  desc, short varlist
  local sortvars `r(sortlist)'
  if "`sortvars'" != "" {
    tempvar order
    gen `order' = _n
  }

	// This section identifies overlapping prescriptions
	bys patid prodcode (start): gen overlap = start < real_stop[_n-1] & real_stop[_n-1] < .
	bys patid prodcode (start): replace overlap = 1 if start == start[_n-1]
	bys patid prodcode (start): replace overlap = 1 if start == real_stop[_n-1] & start == real_stop
	gen duration = real_stop - start
	
	// Add overlap period at next available gap. This is tricky: there is no guarantee
	// that the gap will be long enough, and adding extra records is tricky in stata
	// Therefore, we add one day at a time, then merge groups of days where possible

	// Generates variable of original start date
	gen rstart = start
	// splits rows where there are overlaps
	tvc_split start real_stop, common(patid prodcode) 
	// Note: This produces a number of rows with the same start and end dates but first row has a
  // different prescription id
	if "`dec9'" == "9a" {
		// sum ndd for each time period, drop extra records
		sort patid prodcode start
		by patid prodcode start: egen sum_ndd = sum(ndd) if ndd !=.
		by patid prodcode start: replace sum_ndd = max_rec_ndd if sum_ndd > max_rec_ndd & sum_ndd < .
		by patid prodcode start: replace ndd = sum_ndd if _N > 1
    by patid prodcode start: gen overlapped = _N > 1
    label variable overlapped "Whether this time period corresponds to the overlap between multiple prescriptions."
		by patid prodcode start: drop if _n > 1
    drop sum_ndd
	}
	
	else if "`dec9'" == "9b" {
	
    tostring patid, gen(patid_string)
    format start real_stop %dD/N/CY
    // recalculates duration after tvc_split
    replace duration = real_stop - start
    //table prodcode, c(sum duration)
	
    sort patid_string prodcode start presc_id
    // Identifies number of days overlap and therefore number of rows to be added, ignoring different prescription ids
    by patid_string prodcode start: gen expand     = ceil(duration) + 1 if _n > 1
    // Indicator to 1 (and not to expand) for all other rows
    by patid_string prodcode start: replace expand = 1 if _n == 1
    expand expand
	
    sort patid_string prodcode start presc_id
    // Created new duration which is the rounded down duration if duration has a decimal point and it has been expanded (none in sample)
    by patid_string prodcode start presc_id: gen nduration = duration - floor(duration) if _n == _N & duration > floor(duration) & expand > 1
    // replaces duration to 1 if row is an expanded record (expansion adds a row for each day overlap)
    by patid_string prodcode start presc_id: replace nduration = 1 if expand > 1 & nduration == . & _n > 1
    drop expand
    // Indicators of expanded rows
    by patid_string prodcode start presc_id: gen todo     = _n > 1
    by patid_string prodcode start presc_id: gen expanded = _n > 1
    by patid_string prodcode start presc_id: gen final    = expanded
    // Updates final so first row of expanded prescription id is zero
    by patid_string prodcode start: replace final = 1 if _n == 1
    local last_rec = _N
    // i is the index of the current record where they may be a gap
    local i = 1
    // j is the index of the first overlap that needs to be added
    local j = -1
    // patid is patid on i row
    local patid       = patid_string[1]
    // prodcode is prodcode on i row
    local prodcode    = prodcode[1]
    // start is start on  i row
    local start       = start[1]
    // nstart is start on i row
    local nstart      = `start'
    // while row is less than or equal to the last row
    while `i' <= `last_rec' {
      quietly {
        // New prodcode: update last one and reset locals
        // Patid, prodcode and start macros updated below when moves onto a new patid
        // If the patid on row is not equal to patid current patid in patid macro, or prodcode is not equal prodcode in prodcode macro
        // this will be true when it moves onto the next patients, and patid macro will be updated
        if patid_string[`i'] != "`patid'" | prodcode[`i'] != `prodcode' {
          di "`i' `j' ""`patid'"" `prodcode' "
          // If j is less than ith row does not equal -1
          if `j' < `i' & `j' > -1 {
            // we have not added all overlaps: finish adding them before
            // dealing with next prodcode / patid
            // set nstart to the day after the last prescription for the previous person
            //local nstart = real_stop[`i'-1]
            local lstart = real_stop[`i'-1]
            if `lstart' > `nstart' /*| patid[`i']!=patid[`i'-1] */{
              local nstart = `lstart'
              di `nstart'
						}
            // If j is less than or equal to row number 
            while `j' <= `i' {
              replace start     = `nstart' in `j'
              replace real_stop = `nstart' + nduration in `j'
              replace todo      = 0 in `j'
              di start real_stop todo
              while todo[`j'] == 0 & `j' <= `i' {
                local j = `j' + 1
                di `j'
              }
              local nstart = `nstart' + 1
              local nstart_pat = patid_string[`j']
              di `nstart'
            }				
          }
          local patid       = patid_string[`i']
          local prodcode    = prodcode[`i']
          local start       = start[`i']
          local j           = -1
				}
        // If patid is equal to patid on row
        else {
          // Checks if `j' is -1 (i.e. its the first row that needs updating)
          local nstart = real_stop[`i']
          if `j' == -1 & todo[`i'] == 1 {
            // j updated to row number that needs updating
            local j = `i'
            di `j'
          }
          // Checks if j is greater than -1 and there is an overlap to next row ie start in next row 
          // is greater than stop on this row, start on the next row is not missing and prodcode and patid are the same
          // Will not be true on row where overlap identified i.e. j updated to row before change
          if `j' > -1 & start[`i'+1] > real_stop[`i'] & start[`i'+1] < . & ///
              patid_string[`i'+1] == patid_string[`i'] & prodcode[`i'+1] == prodcode[`i']{
            // creates macro of stop date where overlap identified
            //local nstart_pat = `patid'
            di `nstart'
            // Creates maco of start date of rwo after overlap identified
            local nstop  = start[`i'+1]
            di `nstop'
            // If stop of overlap is less than the start date of the row after where the overlap was identified and j does not equal -1
            // while keeps looping over while it is true so it will update more than one row i.e. if overlap is 2 days will loop over twice
            while `nstart' < `nstop' & `j' > -1 {
              // Replace the start date in row that needs updating i.e. jth row identified earlier
              replace start     = `nstart' in `j'
              // Replaces stop date to start plus the nduration value in the row that needs updating
              replace real_stop = `nstart' + nduration in `j'
              // Alters todo indicator to zero once updated
              replace todo      = 0 in `j'
              // Updates j to the next row number if row j has been updated
              while todo[`j'] == 0 & `j' <= `i' {
                local j = `j' + 1
                di `j'
              }
              if `j' > `i' {
                local j = -1
              }
              // Updates nstart to the next date
              local nstart = `nstart' + 1
              local nstart_pat = "`patid'"
            }
          }
        }
        // Updates i to next row number	
        local i = `i' + 1
      }
      if mod(`i', 1000) == 0 {
        noi di "." _continue
      }
      if mod(`i', 50000) == 0 {
        noi di %10.0f `i'
      }
    }

    if `j' < `i' & `j' > -1 {
      // we have not added all overlaps: finish adding them before
      // dealing with next prodcode / patid
      local nstart = real_stop[`i'-1]
      while `j' < `i' {
        replace start     = `nstart' in `j'
        replace real_stop = `nstart' + nduration in `j'
        replace todo      = 0 in `j'
        while todo[`j'] == 0 & `j' < `i' {
          local j = `j' + 1
        }
        local nstart = `nstart' + 1
      }	
    }

    // Now combine multiple contiguous records for a given prescription into a single record
		
    replace duration = real_stop - start
    //	table prodcode if expanded == 0, c(sum duration)
    //	table prodcode if final    == 1, c(sum duration)
		
    sort patid_string prodcode presc_id start
    by patid_string prodcode presc_id: gen drop = real_stop == start[_n+1] & expanded == 1
    by patid_string prodcode presc_id: replace start = start[_n - 1] if start == real_stop[_n-1] ///
        & expanded == 1 & expanded[_n-1] == 1
    by patid_string prodcode presc_id: drop if drop == 1

    replace duration = real_stop - start
    //	table prodcode if expanded == 0, c(sum duration)
    //	table prodcode if final    == 1, c(sum duration)
		
    drop patid_string
    keep if final == 1

    drop todo final
    label variable expanded "This record is an overlapping period of exposure moved to a later time."
	}
  
  drop rstart overlap duration
  replace real_duration = real_stop - start

  if "`sortvars'" != "" {
    sort `sortvars' `order'
    sort `sortvars'
  }
  
  
